@@ -1,4 +1,4 @@
-#!/afs/ece/usr/tcl/bin/wish -f
+#!/afs/ece/usr/tcl/bin/wisha -f
 # Top-level beth calling routine
 
 # To run this program, you should make sure the top line reflects the location
@@ -9,7 +9,6 @@ if {![info exists beth_dir]} {
 	set beth_dir /afs/ece/usr/svoboda/src/beth
 }
 
-#
 #
 # Beth takes any number of arguments. Args consist of options and configurations
 # Each option brings up a separate Beth window. Configs modify all subsequent
@@ -22,12 +21,14 @@ if {![info exists beth_dir]} {
 #	- 		stdin
 #	=		stdin (gradually)
 #	X		X-selection
+#	0		nothing
 # Configs
+#	-w configs	Configs to be passed to text widget. (such as -height)
 #	-c 		controls input (via a global grab)
 #	-i 		iconified (overrides -c)
 #	-b 		browse (instead of edit)
 #	-t title	(to add to current window title)
-#
+#	-l script	Load script (which lives in either pwd or beth_dir)
 #
 
 # Initial .beth file locations
@@ -52,7 +53,7 @@ set other_configs ""
 set i 0
 while {$i < $argc} {
 	set item [lindex $argv $i]
-	if {($item == "-t")} {
+	if {($item == "-t") || ($item == "-l") || ($item == "-w")} {
 		incr i
 		lappend other_configs $item [lindex $argv $i]
 	} elseif {[string match -? $item]} {
@@ -67,29 +68,42 @@ while {$i < $argc} {
 }
 
 if {($option == "") && ![info exists embedded]} {
-	puts {Beth options: [-c] [-i] [-b] [-t title] [X - = file |cmd |cmd&]}
+	puts \
+{Beth options: [-c] [-i] [-b] [-t title] [-l script] [-w tcl-text-configs]
+               [0 X - = file |cmd |cmd&]}
 	exit 1
 }
 
 # Tell user what is being instantiated.
 # puts "[pid]: beth $configs $option"
 
-# Instantiate configs.
-if {([string first -i $configs] >= 0)} {wm iconify .
-} elseif {([string first -c $configs] >= 0)} {after 1000 {update ; take_control $frame}}
-if {([set v [lsearch $configs -t]] >= 0)} {
-	set title_comment [lindex $configs [incr v]]
-} else {set title_comment {}
+proc return_option_assocs {option list} {
+	set result ""
+	set append_flag 0
+	foreach item $list {
+		if $append_flag {set result [lappend result $item]}
+		set append_flag 0
+		if {$item == $option} {set append_flag 1}
+	}
+	return $result
 }
 
+# Instantiate configs.
+if {([lsearch $configs -i] >= 0)} {wm iconify .
+} elseif {([lsearch $configs -c] >= 0)} {after 1000 {update ; take_control $frame}}
+set title_comment [return_option_assocs "-t" $configs]
+set load_modules [return_option_assocs "-l" $configs]
+set text_configs [return_option_assocs "-w" $configs]
 set cwd [pwd]
 
 # Get text and entry widgets
+source $beth_dir/strings.tcl
+source $beth_dir/bindings.tcl
 source $beth_dir/widgets.tcl
 source $beth_dir/entrybind.tcl
 
 # Alter look if we are editing vs browsing
-if {([string first -b $configs] >= 0)} {
+if {([lsearch $configs -b] >= 0)} {
 	set edit_flag 0} else {set edit_flag 1}
 
 # Name of this interp
@@ -105,6 +119,7 @@ if {($edit_flag) && [file exists $option] && ![file writable $option]} {
 source $beth_dir/dialog.tcl
 source $beth_dir/browsebind.tcl
 source $beth_dir/windowbind.tcl
+source $beth_dir/commandbind.tcl
 source $beth_dir/searchbind.tcl
 source $beth_dir/interpbind.tcl
 source $beth_dir/keybind.tcl
@@ -114,16 +129,34 @@ if {$edit_flag} {
 	source $beth_dir/replacebind.tcl
 }
 
-if {[file exists $config_dir1/$config_file]} {source $config_dir1/$config_file}
-if {($config_dir1 != $config_dir2) &&
-    [file exists $config_dir2/$config_file]} {
-	source $config_dir2/$config_file}
+foreach module $load_modules {
+	global cwd beth_dir
+	if {[file exists $cwd/$module]} {source $cwd/$module
+	} elseif {[file exists $beth_dir/lib/$module]} {source $beth_dir/lib/$module
+}}
 
-proc set_start_title {wm_name} {
+# sources file in either $config_dir1 or $config_dir2
+proc source_config_file {file} {
+	global config_dir1 config_dir2
+	if {[file exists $config_dir1/$file]} {
+		uplevel #0 source $config_dir1/$file}
+	if {($config_dir1 != $config_dir2) &&
+	    [file exists $config_dir2/$file]} {
+		uplevel #0 source $config_dir2/$file}
+}
+source_config_file $config_file
+
+proc set_window_title {wm_name} {
 	global edit_flag title_comment
+	if {$title_comment != ""} {
+		set title_comment [string trim [lindex $title_comment  0] { }]
+		set title_comment "$title_comment "}
 	if {$edit_flag} {
-		wm title . "Edit: $wm_name $title_comment" ; wm iconname . "E $wm_name $title_comment"
-	} else {wm title . "Browse: $wm_name $title_comment" ; wm iconname . "B $wm_name $title_comment"
+		wm title . "$title_comment\Edit: $wm_name"
+		wm iconname . "$title_comment\E $wm_name"
+
+	} else {wm title . "$title_comment\Browse: $wm_name"
+		wm iconname . "$title_comment\B $wm_name"
 }}
 
 
@@ -132,21 +165,24 @@ if {($option == "-")} {
 	set path $cwd
 	set name ""
 	append wm_name $option
-	if {![info exists dont_change_title]} {set_start_title $wm_name}
-	after 0 beth_busy $text read_file_handler $text stdin 0
+	if {![info exists dont_change_title]} {set_window_title $wm_name}
+	after 0 beth_busy $text read_file_handler $text end stdin 0 \; $text mark set insert 1.0
 } elseif {($option == "=")} {
 	set path $cwd
 	append wm_name $option
-	if {![info exists dont_change_title]} {set_start_title $wm_name}
-	after 0 beth_busy $text read_file_handler $text stdin 1
+	if {![info exists dont_change_title]} {set_window_title $wm_name}
+	after 0 beth_busy $text read_file_handler $text end stdin 1 \; $text mark set insert 1.0
 } elseif {($option == "X")} {
 	set path $cwd
 	append wm_name $option
-	if {![info exists dont_change_title]} {set_start_title $wm_name}
+	if {![info exists dont_change_title]} {set_window_title $wm_name}
 	if {[catch {$text insert insert "[selection get]"}]} {
 		flash_label $frame -text "Cannot get X selection"
 	}
 	$text mark set insert 1.0
+} elseif {($option == "0")} {
+		set path $cwd
+		set name ""
 } elseif {($option != "")} {
 	if {([string match \|* $option])} {
 		set name $option
@@ -157,9 +193,9 @@ if {($option == "-")} {
 		} else {set path $cwd/[file dirname $option]}
 	}
 	append wm_name $name
-	if {![info exists dont_change_title]} {set_start_title $wm_name}
+	if {![info exists dont_change_title]} {set_window_title $wm_name}
 	if {[file tail $path] == "."} {set path [file dirname $path]}
-	after 0 load_file $text $frame \"$path\" \"$name\"
+	after 0 load_file $text end $frame \"$path\" \"$name\" \; $text mark set insert 1.0
 } else {set dont_change_title 1
 	set path $cwd
 }
@@ -170,14 +206,12 @@ catch {set fsBox(name) $name}
 catch {set fsBox(path) $path}
 
 # Default generic shell command
-set shell_cmd ""
+set sh_cmd ""
 # Make command
 set make_cmd "$beth_exec \"| make |& cat &\" &"
 # Print command
 set print_cmd "| lpr -m -p -T [lindex [wm title .] 1]"
 # Cmd to browse the X region
-set browse_cmd "| $beth_exec - &"
+set browse_cmd "| $beth_exec -t {X-region} - &"
 # Cmd to view help
 set help_cmd "$beth_exec -b $beth_dir/HELP &"
-
-return 0
