@@ -38,11 +38,13 @@ proc select_all {t} {
 
 # Cursor movement code
 
-# Moves insert specified by index.
+# Moves insert specified by index. Returns 1 if successful.
 proc move_insert {t index} {
-	if {[$t compare insert == $index]} {beep}
+	if {[catch {$t compare insert == $index} flag]} {beep ; return 0}
+	if $flag {beep ; return 0}
 	$t mark set insert $index
 	$t yview -pickplace insert
+	return 1
 }
 
 # Used to naintain column while traversing across lines that are too short.
@@ -126,8 +128,8 @@ proc goto_where {t l e} {
 proc goto {t f} {
 	create_f_entry $t $f.gotol $f.gotoe
 	$f.gotol configure -text "Goto:"
-	bind $f.gotoe <Return> "goto_where $t $f.gotol $f.gotoe"
-	bind $f.gotoe <KP_Enter> "[bind $f.gotoe <Return>]"
+	global Keys
+	parse_bindings $f.gotoe $Keys(C_m) "goto_where $t $f.gotol $f.gotoe"
 }
 
 proc quit_beth {} {
@@ -137,65 +139,68 @@ proc quit_beth {} {
 }
 
 
-# Browse bindings. f is a frame widget to put messages in.
-# q is a button that exits.
-proc browsebind {f} {
-	bind Text <Key> {if {[regexp . %A]} {beep}}
-	bind Text <Return> {beep}
-	bind Text <KP_Enter> {beep}
-	bind Text <Delete> {beep}
-	bind Text <Control-Key> {if {[regexp . %A]} {beep}}
-	bind Text <Meta-Key> {if {[regexp . %A]} {beep}}
-	bind Text <Control-Meta-Key> {if {[regexp . %A]} {beep}}
+# Browse bindings.
+proc browsebind {f t m} {
+	global Keys
+	parse_bindings Text \
+C-space			{%W mark set mark insert} \
+$Keys(C_a)		{move_insert %W {insert linestart}} \
+$Keys(C_b) 		{move_insert %W {insert -1 chars}} \
+M-b 			{move_insert %W {insert -1 chars wordstart}} \
+C-c 			{%W tag add sel 1.0 ; %W tag remove sel 1.0 end} \
+$Keys(C_e)		{move_insert %W {insert lineend}} \
+$Keys(C_f)		{move_insert %W {insert +1 chars}} \
+M-f			{move_insert %W {insert wordend}} \
+{C-K C-k}		{select_next_line %W} \
+C-l			{center_cursor %W} \
+"$Keys(C_W) C-w"	{select_region %W} \
+"$Keys(M_w)" 		{select_all %W} \
+C-x 			{exchange_dot_and_mark %W} \
+$Keys(M_comma) 		{move_insert %W @0,0} \
+$Keys(M_period)		{end_of_page %W}
 
-	bind Text <Control-space> "%W mark set mark insert"
-	bind Text <Control-a> "move_insert %W {insert linestart}"
-	bind Text <Control-b> "move_insert %W {insert -1 chars}"
-	bind Text <Meta-b> "move_insert %W {insert -1 chars wordstart}"
-	bind Text <Control-c> "%W tag add sel 1.0 ; %W tag remove sel 1.0 end"
-	bind Text <Control-e> "move_insert %W {insert lineend}"
-	bind Text <Control-f> "move_insert %W {insert +1 chars}"
-	bind Text <Meta-f> "move_insert %W {insert wordend}"
-	bind Text <Meta-g> "goto %W $f"
-	bind Text <Control-g> "beep ;
-				catch \{destroy_f_entry %W $f.gotol $f.gotoe\}"
-	bind Text <Control-K> "select_next_line %W"
-	bind Text <Control-l> "center_cursor %W"
-	bind Text <Control-n> "adjacent_line %W +1"
-	bind Text <Control-p> "adjacent_line %W -1"
-	bind Text <Meta-q> "quit_beth"
-	bind Text <Control-v> "page_down %W"
-	bind Text <Meta-v> "page_up %W"
-	bind Text <Control-W> "select_region %W"
-	bind Text <Meta-w> "select_all %W"
-	bind Text <Control-x> "exchange_dot_and_mark %W"
-	bind Text <Control-z> "%W yview {@0,0 +1 lines}
-			ensure_cursor_on_screen %W"
-	bind Text <Meta-z> "%W yview {@0,0 -1 lines}
-			ensure_cursor_on_screen %W"
-	bind Text <Meta-comma> "move_insert %W @0,0"
-	bind Text <Meta-period> "end_of_page %W"
-	bind Text <Meta-less> "move_insert %W 1.0"
-	bind Text <Meta-greater> "move_insert %W end"
+	parse_bindings all \
+"$Keys(C_m) Delete" 	{beep} \
+$Keys(A_Key)		{if {[regexp . %A]} {beep}} \
+M-g			"goto $t $f" \
+C-g 			"+ catch \{destroy_f_entry $t $f.gotol $f.gotoe\}" \
+$Keys(C_n) 		"adjacent_line $t +1" \
+$Keys(C_p) 		"adjacent_line $t -1" \
+M-q 			{quit_beth} \
+"$Keys(C_v) space" 	"page_down $t" \
+$Keys(M_v) 		"page_up $t" \
+C-z	 	"$t yview {@0,0 +1 lines} ; ensure_cursor_on_screen $t" \
+M-z 		"$t yview {@0,0 -1 lines} ; ensure_cursor_on_screen $t" \
+$Keys(M_less)	 	"move_insert $t 1.0" \
+$Keys(M_greater) 	"move_insert $t end"
 
-# Duplicate bindings
-	bind Text <Control-k> [bind Text <Control-K>]
-	bind Text <Control-w> [bind Text <Control-W>]
-	bind Text <space> [bind Text <Control-v>]
-	bind Text <Prior> [bind Text <Meta-v>]
-	bind Text <Shift-Prior> [bind Text <Meta-less>]
-	bind Text <Next> [bind Text <Control-v>]
-	bind Text <Shift-Next> [bind Text <Meta-greater>]
-	bind Text <Up> [bind Text <Control-p>]
-	bind Text <Shift-Up> [bind Text <Meta-comma>]
-	bind Text <Down> [bind Text <Control-n>]
-	bind Text <Shift-Down> [bind Text <Meta-period>]
-	bind Text <Left> [bind Text <Control-b>]
-	bind Text <Shift-Left> [bind Text <Control-a>]
-	bind Text <Right> [bind Text <Control-f>]
-	bind Text <Shift-Right> [bind Text <Control-e>]
-	bind Text <Select> [bind Text <Control-w>]
-	bind Text <Shift-Select> [bind Text <Meta-w>]
-}
+	if {[winfo exists $m]} {parse_menu $m \
+{Browse 0	{Move 0 "" 	{Character 0 ""	{Back 0 C-b}
+						{Forward 0 C-f}
+						{Up 1 C-p}
+						{Down 3 C-n}}
+				{Word 0	""	{Back 0 M-b}
+						{Forward 0 M-f}}
+				{Edge 0	""	{Left 0 C-a}
+						{Right 0 C-e}
+						{Top 0 {M-comma {,}}} 
+						{Bottom 0 {M-period {.}}}}}
+		{Scroll 0 ""	{Line 0	""	{Up 0 M-z}
+						{Down 0 C-z}}
+				{Page 0	""	{Up 0 M-v}
+						{Down 0 C-v}}
+				{Document 0 ""	{Beginning 0 {M-less {<}}}
+						{End 0 {M-greater {>}}}}}
+		{Select	1 ""			{Line 0 C-K}
+						{Region 0 C-W}
+						{All 0 M-w}
+						{Clear 0 C-c}}
+		{Mark 3	""			{Set 0 {C-space { }}}
+						{Exchange 1  C-x}}
+						{"Center Cursor" 0 C-l}
+						{"Goto Line" 0 M-g}} \
+{File 0						{Quit 0 M-q}}
+}}
 
-browsebind $frame
+
+browsebind $frame $text $menu
