@@ -1,9 +1,8 @@
 # dialog.tcl --
+# Originally the dialog.tcl in the tk library, the dialog has been modified to
+# include keystroke bindings, and a 'listbox' dialog has been added.
 #
-# This file defines the procedure tk_dialog, which creates a dialog
-# box containing a bitmap, a message, and one or more buttons.
-#
-# $Header: /afs/ece/usr/svoboda/src/beth/RCS/dialog.tcl,v 3.1 1994/02/17 21:09:10 svoboda Exp svoboda $ SPRITE (Berkeley)
+# From the original dialog.tcl:
 #
 # Copyright (c) 1992-1993 The Regents of the University of California.
 # All rights reserved.
@@ -59,8 +58,7 @@ proc tk_dialog {w title text bitmap default args} {
 
     # 2. Fill the top part with bitmap and message.
 
-    message $w.msg -width 3i -text $text \
-	    -font -Adobe-Times-Medium-R-Normal-*-180-*
+    message $w.msg -width 3i -text $text
     pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 5m -pady 5m
     if {$bitmap != ""} {
 	label $w.bitmap -bitmap $bitmap
@@ -115,3 +113,77 @@ proc tk_dialog {w title text bitmap default args} {
     focus $oldFocus
     return $tk_priv(button)
 }
+
+# Like tk_dialog, but instead of a dialog of choices, brings up a listbox
+proc tk_dialog_listbox {w title text options {view 0}} {
+    global tk_priv
+
+    # 1. Create the top-level window and divide it into top
+    # and bottom parts.
+
+    catch {destroy $w}
+    toplevel $w -class Dialog
+    wm title $w $title
+    wm iconname $w Dialog
+    frame $w.top -relief raised -bd 1
+    pack $w.top -side top -fill both
+    frame $w.bot -relief raised -bd 1
+    pack $w.bot -side bottom -fill both
+
+    message $w.msg -width 3i -text $text
+    pack $w.msg -in $w.top -side top -expand 1 -fill both -padx 5m -pady 5m
+
+    listbox $w.top.list -relief sunken -geometry 20x5
+    scrollbar $w.top.hscroll -orient {horizontal} -relief {raised}
+    scrollbar $w.top.vscroll -relief {raised}
+    $w.top.vscroll configure -command "$w.top.list yview"
+    $w.top.hscroll configure -command "$w.top.list xview"
+    $w.top.list configure -xscrollcommand "$w.top.hscroll set" \
+			-yscrollcommand "$w.top.vscroll set"
+    tk_listboxSingleSelect $w.top.list
+    pack $w.top.hscroll -in $w.top -side bottom -fill x
+    pack $w.top.list -in $w.top -side right -fill both -expand yes
+    pack $w.top.vscroll -in $w.top -side right -fill y
+    eval $w.top.list insert 0 [lsort $options]
+    bind $w.top.list <Double-ButtonPress-1> {set tk_priv(button) [%W nearest %y]}
+    incr view -10 ; if {$view < 0} {set view 0}
+    $w.top.list xview $view
+
+    button $w.button -text "OK" -command "set tk_priv(button) -1"
+    bind $w <Key-1> "$w.button flash; set tk_priv(button) -1"
+    frame $w.default -relief sunken -bd 1
+    raise $w.button $w.default
+    pack $w.default -in $w.bot -side left -expand 1 -padx 3m -pady 2m
+    pack $w.button -in $w.default -padx 2m -pady 2m -ipadx 2m -ipady 1m
+    bind $w <Return> "$w.button flash; set tk_priv(button) -1"
+
+    # 4. Withdraw the window, then update all the geometry information
+    # so we know how big it wants to be, then center the window in the
+    # display and de-iconify it.
+
+    wm withdraw $w
+    update idletasks
+    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
+	    - [winfo vrootx [winfo parent $w]]]
+    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
+	    - [winfo vrooty [winfo parent $w]]]
+    wm geom $w +$x+$y
+    wm deiconify $w
+
+    # 5. Set a grab and claim the focus too.
+
+    set oldFocus [focus]
+    grab $w
+    focus $w
+
+    # 6. Wait for the user to respond, then restore the focus and
+    # return the index of the selected button.
+
+    set selection ""
+    tkwait variable tk_priv(button)
+    if {$tk_priv(button) == -1} {set result ""
+    } else {set result [$w.top.list get $tk_priv(button)]}
+    destroy $w
+    focus $oldFocus
+    return $result
+}
