@@ -56,10 +56,19 @@ proc e_exchange_dot_and_mark {e} {
 
 # Editing Selection
 
+# Removes contents of e, and replaces with new_string, leaving view intact.
+proc e_replace {e new_string} {
+	set new_index [expr [$e index insert] + [string length $new_string] - \
+				[string length [$e get]]]
+	set beginning [$e index @0]
+	$e delete 0 end
+	$e insert 0 $new_string
+	$e view $beginning
+	$e icursor $new_index
+}
+
 proc e_kill_next_line {e} {
-	.e_kill delete 0 end
-	.e_kill insert 0 [string range [$e get] [$e index insert] \
-						[$e index end]]
+	e_replace .e_kill [string range [$e get] [$e index insert] [$e index end]] 
 	$e delete insert end
 	.e_kill select from 0
 	.e_kill select to end
@@ -68,14 +77,12 @@ proc e_kill_next_line {e} {
 proc e_kill_region {e} {
 	global e_mark
 	if {($e_mark < 0)} {beep ; return}
-	.e_kill delete 0 end
 	if {[$e index insert] > $e_mark} {
 		set start $e_mark
 		set end insert
 	} else {set start insert
 		set end $e_mark}
-	.e_kill insert 0 [string range [$e get] [$e index $start] \
-						 [$e index $end]]
+	e_replace .e_kill [string range [$e get] [$e index $start] [$e index $end]]
 	.e_kill select from 0
 	.e_kill select to end
 	$e delete $start $end
@@ -109,22 +116,23 @@ proc e_backspace {e} {
 }}
 
 proc e_unquote {e c} {
+	global Keys
 	$e insert insert $c
-	bind $e <Key> ""
-	bind $e <Control-Key> ""
+	parse_bindings $e $Keys(A_C_Key) {}
 	e_recenter $e
 }
 
 proc e_key_quote {e c} {
+	global Keys
 	if {([regexp . $c])} {
 		beep
-		bind $e <Key> ""
-		bind $e <Control-Key> ""
+		parse_bindings $e $Keys(A_C_Key) {}
 }}
 
 proc e_quote_insert {e} {
-	bind $e <Key> "e_key_quote $e %A"
-	bind $e <Control-Key> "e_unquote $e %A"
+	parse_bindings $e \
+Key				"e_key_quote $e %A" \
+C-Key				"e_unquote $e %A"
 }
 
 proc e_transpose_chars {e} {
@@ -166,9 +174,12 @@ proc e_transpose_words {e} {
 	e_recenter $e
 }
 
+set overwrite_mode 0
+
 proc e_self_insert {e {c ""}} {
 	if {(![regexp . $c])} {return}
 	$e insert insert $c
+	global overwrite_mode ;	if $overwrite_mode {$e delete insert}
 	e_recenter $e
 }
 
@@ -227,6 +238,8 @@ proc e_delete_word_forward {e} {
 	e_recenter $e
 }
 
+
+# Filters that replace one string with another
 proc string_tolower {s} {return [string tolower $s]}
 proc string_toupper {s} {return [string toupper $s]}
 proc string_capitalize {s} {
@@ -259,65 +272,137 @@ proc e_filter_word {e filter} {
 }
 
 
-# Entry bindings
-proc entrybind {} {
-	bind Entry <Key> {e_self_insert %W %A}
-	bind Entry <Return> {beep}
-	bind Entry <Control-Key> {if {[regexp . %A]} {beep}}
-	bind Entry <Meta-Key> {if {[regexp . %A]} {beep}}
-	bind Entry <Control-Meta-Key> {if {[regexp . %A]} {beep}}
-	bind Entry <Escape> {;}
+# String replacement / completion
 
-	bind Entry <Control-Delete> "e_delete_selection %W"
-	bind Entry <Control-space> {set e_mark [%W index insert]}
-	bind Entry <Control-a> "%W icursor 0 ; e_recenter %W"
-	bind Entry <Control-b> "e_move_char %W -1"
-	bind Entry <Meta-b> {e_back_word %W}
-	bind Entry <Control-c> "%W select from 0 ; %W select to end ; %W select clear"
-	bind Entry <Meta-c> "e_filter_word %W string_capitalize"
-	bind Entry <Control-d> "e_delete_char %W"
-	bind Entry <Meta-d> "e_delete_word_forward %W"
-	bind Entry <Control-e> "%W icursor end ; e_recenter %W"
-	bind Entry <Control-f> "e_move_char %W +1"
-	bind Entry <Meta-f> {e_forward_word %W}
-	bind Entry <Control-d> "e_delete_char %W"
-	bind Entry <Control-g> {eval [bind Text <Control-g>]}
-	bind Entry <Control-h> "e_backspace %W"
-	bind Entry <Meta-h> "e_delete_word_back %W"
-	bind Entry <Control-i> {e_self_insert %W \t}
-	bind Entry <Control-k> "e_kill_next_line %W"
-	bind Entry <Control-K> "e_select_next_line %W"
-	bind Entry <Control-l> "e_recenter %W"
-	bind Entry <Meta-l> "e_filter_word %W string_tolower"
-	bind Entry <Control-q> "e_quote_insert %W"
-	bind Entry <Control-t> "e_transpose_chars %W"
-	bind Entry <Meta-t> "e_transpose_words %W"
-	bind Entry <Control-u> "%W icursor 0 ; e_kill_next_line %W"
-	bind Entry <Meta-u> "e_filter_word %W string_toupper"
-	bind Entry <Control-w> "e_kill_region %W"
-	bind Entry <Control-W> "e_select_region %W"
-	bind Entry <Meta-w> "%W select from 0 ; %W select to end"
-	bind Entry <Control-x> "e_exchange_dot_and_mark %W"
-	bind Entry <Control-y> "e_paste_selection %W"
-	bind Entry <Control-bracketright> "widget_resize %W -width -1"
-	bind Entry <Control-braceright> "widget_resize %W -width +1"
-
-# Duplicate Keybindings
-	bind Entry <Key-KP_Enter> "[bind Entry <Return>]"
-	bind Entry <Delete> "[bind Entry <Control-h>]"
-	bind Entry <Select> [bind Entry <Control-W>]
-	bind Entry <Shift-Select> [bind Entry <Meta-w>]
-	bind Entry <Left> [bind Entry <Control-b>]
-	bind Entry <Shift-Left> [bind Entry <Control-a>]
-	bind Entry <Right> [bind Entry <Control-f>]
-	bind Entry <Shift-Right> [bind Entry <Control-e>]
-	bind Entry <Insert> [bind Entry <Control-y> ]
-	bind Entry <Control-v> [bind Entry <Control-y> ]
-	bind Entry <ButtonRelease-3> [bind Entry <Control-y>]
-	bind Entry <Button1-Delete> [bind Entry <Control-Delete>]
-	bind Entry <Control-Left> [bind Entry <Control-bracketright>]
-	bind Entry <Control-Right> [bind Entry <Control-braceright>]
+# Does completion to the string in an entry binding.
+# Beeps unless exact completion was made.
+proc e_complete_string {e possibilities {message_prelude "Possible values:\n"} \
+				{function filter_completions}} {
+	set completion [complete_string $function $possibilities [$e get]]
+	global completion_message completion_index
+	set completion_index -1
+	if {[lindex $completion 1] != ""} {
+		set completion_message [lindex $completion 1]
+	} else {set completion_message ""}
+	if {([lindex $completion 0] == "") || ([lindex $completion 1] != "")} {beep}
+	if {[lindex $completion 0] != ""} {e_replace $e [lindex $completion 0]}
 }
 
-entrybind
+# Does completion to the substring in an entry binding before the insert cursor.
+# Beeps unless completion was made and no message exists.
+proc e_complete_multiple {e {completion_list ""}} { 
+	set index [expr [$e index insert] - 1]
+	set head [string range [$e get] 0 $index]
+	set tail [string range [$e get] [$e index insert] end]
+	set completion [multiple_completion $head $completion_list]
+	global completion_message completion_index
+	set completion_message [lindex $completion 1]
+	set completion_index [lindex $completion 2]
+	if {([lindex $completion 0] == "") || ([lindex $completion 1] != "")} {beep}
+	if {[lindex $completion 0] != ""} {
+		e_replace $e "[lindex $completion 0]$tail"
+}}
+
+# Shows list of completions, letting user pick one.
+proc e_show_completion_dialog {e} {
+	global completion_message completion_index
+	if {$completion_message == ""} {beep ; return}
+	if {[string match {Possible *} $completion_message]} {
+		set view [expr [string length [$e get]] - $completion_index]
+		set completion [eval tk_dialog_listbox .conf \
+		   {{Completion Dialog}} [split $completion_message \n] $view]
+		if {$completion != ""} {
+			e_replace $e "[string range [$e get] 0 \
+					$completion_index]$completion"}
+	} else {tk_dialog .conf Error $completion_message info 0 OK
+}}
+
+proc toggle_overwrite_mode {} {
+	global overwrite_mode
+	if $overwrite_mode {set overwrite_mode 0} else {set overwrite_mode 1}
+}
+
+
+# Entry bindings
+proc entrybind {m} {
+	global Keys
+	foreach binding [bind Entry] {
+		if {[string match *Key* $binding]} {
+			bind Entry $binding ""}}
+
+	parse_bindings all \
+C-g				{beep} \
+M-o				{toggle_overwrite_mode}
+	bind Text <Escape>	{switch_focus}
+
+	parse_bindings Entry \
+Escape		{switch_focus} \
+Key		{e_self_insert %W %A} \
+$Keys(C_m)	{beep} \
+Tab		{e_complete_multiple %W} \
+C-Tab		{e_show_completion_dialog %W} \
+$Keys(C_Delete)	{e_delete_selection %W} \
+C-space		{set e_mark [%W index insert]} \
+$Keys(C_a)	{%W icursor 0 ; e_recenter %W} \
+{C-b Left}	{e_move_char %W -1} \
+M-b		{e_back_word %W} \
+C-c		{%W select from 0 ; %W select to end ; %W select clear} \
+M-c		{e_filter_word %W string_capitalize} \
+C-d		{e_delete_char %W} \
+M-d		{e_delete_word_forward %W} \
+$Keys(C_e)	{%W icursor end ; e_recenter %W} \
+$Keys(C_f)	{e_move_char %W +1} \
+M-f		{e_forward_word %W} \
+$Keys(C_h)	{e_backspace %W} \
+M-h		{e_delete_word_back %W} \
+C-k		{e_kill_next_line %W} \
+C-K		{e_select_next_line %W} \
+C-l		{e_recenter %W} \
+M-l		{e_filter_word %W string_tolower} \
+C-q		{e_quote_insert %W} \
+C-t		{e_transpose_chars %W} \
+M-t		{e_transpose_words %W} \
+C-u		{%W icursor 0 ; e_kill_next_line %W} \
+M-u		{e_filter_word %W string_toupper} \
+C-w		{e_kill_region %W} \
+{C-W Select}	{e_select_region %W} \
+{M-w S-Select}	{%W select from 0 ; %W select to end} \
+C-x		{e_exchange_dot_and_mark %W} \
+$Keys(C_y)	{e_paste_selection %W}
+
+	if {[winfo exists $m]} {parse_menu $m \
+{Edit 0	{Kill 0	""			{Line 0 C-k}
+					{Region 0 C-w}}
+					{Yank 0 C-y}
+	{Delete 0 ""	{Character 0 ""	{Previous 0 C-h}
+					{Next 0 C-d}}
+			{Word 0	""	{Previous 0 M-h}
+					{Next 0 M-d}}
+					{Region 0 C-Delete}}
+	{Case 0 ""			{Upper 0 M-u}
+					{Lower 0 M-l}
+					{Capitalize 0 M-c}}
+					{Quote 0 C-q}
+	{Transpose 0 ""			{Characters 0 C-t}
+					{Words 0 M-t}}} \
+{Entry 1			{"Switch Focus" 0 Escape}
+				{"Abort" 0 C-g}
+				{"Exit" 0 Return}
+	separator
+				{"Complete Word" 0 Tab}
+				{"Show Completion Info" 16 C-Tab}
+	separator
+				{Reset 0 C-u}}
+
+# No command, since the command merely changes the variable.
+		$m.edit.m add checkbutton -label Overwrite -underline 1 \
+			-variable overwrite_mode -offvalue 0 -onvalue 1 \
+			-accelerator M-o
+
+		$m.entry configure -state disabled
+		$m.edit configure -state disabled
+}}
+
+
+entrybind $menu
 catch {entry .e_kill}
