@@ -1,32 +1,33 @@
 # Bindings for file I/O
 
-
 # Opens up $path/$name or just $name if it is a command pipeline, and
 # dumps contents of text widget t into it.
 proc write_file {t f path name} {
-	if {([string match \|* $name])} {
-		set file [open $name" w]
-	} else {set file [open $path/$name w]}
+	global use_pipe
+	if {([string match \|* $name])} {	set file [open $name w]
+	} elseif $use_pipe {			set file [open "| $name" w]
+	} else {				set file [open $path/$name w]}
 	puts $file [$t get 1.0 end] nonewline
 	close $file
 }
 
 proc default_file_prompter {msg} {
-	global file_done fsBox text frame
+	global file_done fsBox text frame Keys
 	set name $fsBox(name)
-	create_f_entry $text $frame.filel $frame.filee
+	create_f_entry $text $frame.filel $frame.filee {complete_string filter_glob ""}
 	$frame.filel configure -text $msg
 	if {(![string match \|* $name])} {$frame.filee insert 0 $fsBox(path)/}
 	set index [$frame.filee index insert]
 	$frame.filee insert $index $name
 	$frame.filee icursor $index
 	$frame.filee view [expr $index - [lindex [$frame.filee configure -width] 4]]
-	bind $frame.filee <Return> "set file_done 1"
-	bind $frame.filee <KP_Enter> "[bind $frame.filee <Return>]"
-	bind $frame.filee <Control-g> "set file_done 2"
+	parse_bindings $frame.filee \
+$Keys(C_m)	 		{set file_done 1} \
+C-g				{set file_done 2}
 	global file_done cwd;	set file_done 0
 	tkwait variable file_done
-	set what [$frame.filee get]
+	set what [string trim [$frame.filee get] { }]
+
 	destroy_f_entry $text $frame.filel $frame.filee
 	if {$file_done == 2} {
 		beep
@@ -87,8 +88,7 @@ proc save_file {t f} {
 	write_file $t $f $fsBox(path) $fsBox(name)
 	global modified atime;	set modified 0
 	catch {set atime [file mtime $fsBox(path)/$fsBox(name)]}
-	global title_comment
-	reset_my_title "Edit: $fsBox(name) $title_comment" "E $fsBox(name) $title_comment"
+	set_window_title $fsBox(name)
 	check_for_duplicate_interps $t $f $fsBox(path) $fsBox(name)
 	return 1
 }
@@ -114,10 +114,9 @@ proc revert_file {t f} {
 		if {[read_file $t $f] == 0} {
 			set modified $old_modified
 			set return 0}}
-	load_file $t $f $fsBox(path) $fsBox(name)
+	load_file $t end $f $fsBox(path) $fsBox(name)
 	catch {set atime [file mtime $fsBox(path)/$fsBox(name)]}
-	global title_comment
-	reset_my_title "Edit: $fsBox(name) $title_comment" "E $fsBox(name) $title_comment"
+	set_window_title $fsBox(name)
 	check_for_duplicate_interps $t $f $fsBox(path) $fsBox(name)
 	$t yview 1.0 ; $t mark set insert 1.0
 	focus $t
@@ -145,10 +144,11 @@ proc visit_file {f} {
 	if {([file_prompt $read_msg $f] == 0)} {return}
 	set int [exec beth $fsBox(path)/$fsBox(name) &]
 	set fsBox(path) $path;	set fsBox(name) $name
-	if {([grab status .] != "none")} {
-		relinquish_control $f 
-		send $int take_control $f
-}}
+#	if {([grab status .] != "none")} {
+#		relinquish_control $f 
+#		check_send $int take_control $f
+#}
+}
 
 # Set modified if file has been modified since we last read/saved it.
 proc check_mtime {} {
@@ -164,49 +164,37 @@ proc check_mtime {} {
 
 # Waits until all interps have titles differing from their interpreter names.
 proc resolve_all_titles {} {
-	global all_titles all_interps
-	set l [llength $all_interps]
-	while 1 {
-		set all_resolved 1
-		set titles $all_titles
-		for {set i 0} {$i < $l} {incr i} {
-			if {([lindex $all_interps $i] == [lindex $all_titles $i])} {
-				set all_resolved 0
-		}}
-		if $all_resolved break
-		if {$all_titles == $titles} {	tkwait variable all_titles
-}}}
+	global all_interps me
+	foreach interp $all_interps {if {$interp != $me} {
+		if {$interp == [send $interp wm title .]} {after 1000
+}}}}
 
 # Ensures no other interp is editing the same file I am editing.
 proc check_for_duplicate_interps {t f path name} {
-	global all_interps all_titles me
-	set interps $all_interps;	set titles $all_titles
-	set my_title [wm title .]
+	# If we're looking at a pipe output, stdin, or
+	# X selection, don't bother.
+	if {[string match \|* $name] || ($name == "-") || \
+		($name == {""}) || ($name == "X") || ($name == "=") ||
+		($name == "")} {return}
 
-	# This seems to help:
-	after 100
-
-	for {set i 0} {$i < [llength $interps]} {incr i} {
-		set him [lindex $interps $i]
-		set his_title [lindex $titles $i]
-		if {($my_title == $his_title)} {
-
-			# Oh...it's just me.
-			if {($me == $him)} {break}
-
-			# If we're looking at a pipe output, stdin, or
-			# X selection, don't bother.
-			if {[string match \|* $name] || ($name == "-") || ($name == "X") || ($name == "=")} {break}
-
-			# Maybe he can only browse. Fine.
-			if {(![send $him "set edit_flag"])} {break}
-
-			# Maybe he's looking in a different directory.
-			if {($path != [send $him "set fsBox(path)"])} {break}
-
-			# Oh well...gotta resolve it.
-			resolve_mutual_edit_conflict $him "$path/[lindex $my_title 1]" $t $f
-}}}
+	global all_interps me
+	foreach him $all_interps {
+		# Oh...it's just me.
+		if {($me == $him)} {continue}
+		# Maybe he's dead
+		if {![check_interp $him]} {continue}
+		# Maybe he's not ready yet.
+		if {[send $him wm title .] == $him} {continue}
+		# Maybe he's looking at a different file.
+		if {[catch {send $him {set fsBox(name)}} result]} {continue}
+		if {($name != $result)} {continue}
+		# Maybe he's looking in a different directory.
+		if {($path != [send $him "set fsBox(path)"])} {continue}
+		# Maybe he can only browse. Fine.
+		if {(![send $him "set edit_flag"])} {continue}
+		# Oh well...gotta resolve it.
+		resolve_mutual_edit_conflict $him "$path/$name" $t $f
+}}
 
 proc resolve_mutual_edit_conflict {him file t f} {
 	global mutual_edit_conflict_msg
@@ -228,18 +216,32 @@ proc quit_beth {} {
 
 
 # File bindings. f is a frame widget to put messages in.
-proc filebind {f} {
-	bind Text <Control-g> "+catch \{destroy_f_entry %W $f.filel $f.filee\}"
-
-	bind Text <Meta-R> "revert_file %W $f"
-	bind Text <Meta-S> "save_file_prompt %W $f"
-	bind Text <Meta-r> "read_file %W $f"
-	bind Text <Meta-s> "save_file %W $f"
-	bind Text <Meta-Control-v> "visit_file $f"
+proc filebind {f t m} {
 	bind . <Any-Enter> "check_mtime"
-}
+	parse_bindings all \
+C-g			"+catch \{destroy_f_entry $t $f.filel $f.filee\}" \
+M-R			"revert_file $t $f" \
+M-S			"save_file_prompt $t $f" \
+M-r			"read_file $t $f" \
+M-s			"save_file $t $f" \
+M-C-v			"visit_file $f"
 
-filebind $frame
+	if {[winfo exists $m]} {
+		parse_menuentries $m.file.m separator
+		$m.file.m add checkbutton -label "Use a Pipe" -underline 6 \
+			-variable use_pipe -offvalue 0 -onvalue 1
+		$m.file.m add checkbutton -label "Gradual output" -underline 0 \
+			-variable gradual_io -offvalue 0 -onvalue 1
+		parse_menuentries $m.file.m {
+				separator
+					{Read 0 M-r}
+					{Revert 1 M-R}
+					{Save 0 M-s}
+					{"Save as" 5 M-C-s}
+					{Visit 0 M-C-v}}
+}}
+
+filebind $frame $text $menu
 
 # Create file path label
 catch {destroy $frame.fpl}
@@ -253,8 +255,14 @@ label_expand_bind $frame.fnl fsBox(name)
 
 pack $frame.fpl $frame.fnl -side left
 
-# Ensure we're not editing file another beth interp is editing.
 if {(![info exists file_prompter])} {set file_prompter default_file_prompter}
-after 1 {resolve_all_titles
+
+
+# Ensure we're not editing file another beth interp is editing.
+after 1 {
+	after 20000 "set registered 1"
+	tkwait variable registered
+	catch {resolve_all_titles}
 	if {[info exists name]} {
-	check_for_duplicate_interps $text $frame $path \"$name\"}}
+                catch {check_for_duplicate_interps $text $frame $path $name} msg
+}}
