@@ -1,7 +1,7 @@
 # Bindings for query/replace
 
 
-if {([bind Text <Control-s>] == "")} {puts "You will also have to load the searchbindings"}
+if {([bind all <Control-s>] == "")} {puts "You will also have to load the searchbindings"}
 
 
 proc undo_replace {t replacements} {
@@ -43,6 +43,7 @@ proc replace_one {t f} {
 	$t mark set $m1 "insert -[string length $replace_string] chars"
 	$t mark set $m2 insert
 	set replacement [list $m1 $m2 $old_stuff]
+	$t tag add was_replaced $m1 $m2
 	lappend all_replacements $replacement
 	lappend all_replacement_marks $m1 $m2
 
@@ -66,65 +67,77 @@ proc exit_replace {t f c} {
 	destroy_f_entry $t $f.r $f.rs
 	global replace_count
 	flash_label $f -text "Replaced $replace_count occurrences."
+	$t tag remove was_replaced 1.0 end
 
 	global all_replacements all_replacement_marks
 	global search_string replace_string
 	register_undoable_cmd $t [list undo_replace $t $all_replacements] "Replace $search_string with $replace_string" $all_replacement_marks
-}
+	global menu ; menuentries_change_state $menu.search.m disabled 8 9 10
+	foreach binding [bind $t] {	bind $t $binding ""
+}} 
 
 # What to display in the Replace label.
 set replace_msg "Change to: "
 
-proc replace_setup {t f} {
+proc replace_setup {t f m} {
 	global replace_msg replace_string replace_count
 	set replace_count 0
 	$f.ss configure -width [string length [$f.ss get]]
 	create_f_entry $t $f.r $f.rs
 	$f.r configure -text $replace_msg
 	$f.rs configure -textvariable replace_string
+	$t tag lower was_replaced
 
-	global all_replacements all_replacement_marks
+	global all_replacements all_replacement_marks Keys
 	set all_replacements {}
 	set all_replacement_marks {}
 
-	bind $f.rs <Control-g> "exit_search $t $f x ; exit_replace $t $f x ; beep"
-	bind $f.rs <Control-r> [bind $f.ss <Control-r>]
-	bind $f.rs <Control-s> [bind $f.ss <Control-s>]
-	bind $f.rs <Shift-Find> [bind $f.ss <Shift-Find>]
-	bind $f.rs <Find> [bind $f.ss <Find>]
-	bind $f.rs <Return> "exit_replace $t $f %A ; exit_search $t $f %A"
-	bind $f.rs <KP_Enter> "[bind $f.rs <Return>]"
-	bind $f.rs <Control-c> "replace_this_one $t $f"
-	bind $f.rs <Control-C> "beth_busy $t replace_to_end $t $f"
-	bind $f.rs <Escape> "focus $f.ss"
-	bind $t <Escape> "focus $f.rs"
-	bind $f.ss <Control-g> "[bind $f.rs <Control-g>]"
-	bind $f.ss <Return> "[bind $f.rs <Control-g>]"
-	bind $f.ss <KP_Enter> "[bind $f.ss <Return>]"
-}
+	parse_bindings $f.ss $Keys(C_m) [bind all <Control-g>]
+	parse_bindings $f.rs \
+$Keys(C_r)		[bind $f.ss <Control-r>] \
+$Keys(C_s)		[bind $f.ss <Control-s>] \
+$Keys(C_m)		"exit_replace $t $f %A ; exit_search $t $f %A" \
+C-c			"replace_this_one $t $f" \
+C-C			"beth_busy $t replace_to_end $t $f"
 
-proc add_replace_bindings {t f} {
-	bind $t <Control-C> "replace_setup $t $f"
-	bind $t <Control-c> "set replace_string {} ; replace_setup $t $f"
-	bind $f.ss <Control-C> "replace_setup $t $f"
-	bind $f.ss <Control-c> "set replace_string {} ; replace_setup $t $f"
+	if {[winfo exists $m]} {
+		menuentries_change_state $m.search.m normal 10
+		menuentries_change_state $m.search.m disabled 9
+}}
+
+proc reset_replace {} {global replace_string ; set replace_string ""}
+
+proc add_replace_bindings {t f m} {
+	global Keys
+	parse_bindings $f.ss \
+C-C			"replace_setup $t $f $m" \
+C-c			"reset_replace ; replace_setup $t $f $m"
+
+	menuentries_change_state $m.search.m normal 8 9
+	bind all <Control-g> "+ menuentries_change_state $m.search.m disabled 8 9"
+	parse_bindings $f.ss \
+$Keys(C_m)	"+ menuentries_change_state $m.search.m disabled 8 9"
 }
 
 
 # Replace bindings. f is a frame widget to put messages in.
-proc replacebind {f} {
-	if {([string first "add_replace_bindings" [bind Text <Control-s>]] \
+proc replacebind {f t m} {
+	global search_bindings
+	if {([string first "add_replace_bindings" [bind all <Control-s>]] \
 		 < 0)} {
-		bind Text <Control-g> "+ catch \{exit_replace %W $f x\}"
-		bind Text <Control-r> "+add_replace_bindings %W $f"
-		bind Text <Control-s> "+add_replace_bindings %W $f"
-		bind Text <Control-R> "+add_replace_bindings %W $f"
-		bind Text <Control-S> "+add_replace_bindings %W $f"
-		bind Text <Control-Meta-s> "+add_replace_bindings %W $f"
+		bind all <Control-g> "+ catch \{exit_replace $t $f x\}"
+		parse_bindings all \
+$search_bindings	"+ add_replace_bindings $t $f $m"}
 
-# Duplicate bindings
-		bind Text <Shift-Find> [bind Text <Control-r>]
-		bind Text <Find> [bind Text <Control-s>]
+		if {[winfo exists $m]} {
+			parse_menuentries $m.search.m {
+				separator
+					{Change 0 C-c}
+					{"Change Previous" 7 C-C}
+					{"Change All" 7 C-C}}
+
+			menuentries_change_state $m.search.m disabled 8 9 10
 }}
 
-replacebind $frame
+
+replacebind $frame $text $menu
